@@ -1,7 +1,7 @@
 # copyright (C) 1997-98 Jean-Luc Fontaine (mailto:jfontain@mygale.org)
-# this program is free software: please refer to the BSD type license enclosed in this package
+# this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: blt2d.tcl,v 1.17 1998/04/30 10:06:45 jfontain Exp $}
+set rcsId {$Id: blt2d.tcl,v 1.24 1998/06/21 12:04:32 jfontain Exp $}
 
 class blt2DViewer {
 
@@ -9,11 +9,12 @@ class blt2DViewer {
     set blt2DViewer::(colors) {#7FFFFF #7FFF7F #FF7F7F #FFFF7F #7F7FFF #FFBF00 #BFBFBF #FF7FFF #FFFFFF}
 
     proc blt2DViewer {this path} viewer {} {
-        $path configure -plotbackground $widget::(default,ButtonBackgroundColor) -cursor {}
+        $path configure -cursor {}
 if {$::officialBLT} {
-        $path configure -plotpadx 1 -plotpady {2 1}                                            ;# leave a little more padding on top
+        $path configure -plotpadx 2 -plotpady 2           ;# use minimum padding for extreme values and flat zero line to be visible
+        $path yaxis configure -tickshadow {}
 }
-        $path yaxis configure -title {} -min 0 -tickfont $font::(smallNormal)
+        $path yaxis configure -title {} -tickfont $font::(smallNormal)
         $path legend configure -borderwidth 1 -font $font::(mediumNormal) -activebackground white
 
         # allow dropping of data cells
@@ -21,22 +22,15 @@ if {$::officialBLT} {
             -path $path -formats DATACELLS -command "viewer::view $this \$dragSite::data(DATACELLS)"\
         ]
 
-        set blt2DViewer::($this,drag) [new dragSite -path $path -validcommand "blt2DViewer::validateDrag $this"]
-        dragSite::provide $blt2DViewer::($this,drag) OBJECTS "blt2DViewer::dragData $this"                ;# provide list of objects
-        dragSite::provide $blt2DViewer::($this,drag) DATACELLS "blt2DViewer::dragData $this"
-
         set blt2DViewer::($this,elements) {}                                             ;# initialize list of data line identifiers
-        set blt2DViewer::($this,selector) [new objectSelector -selectcommand "blt2DViewer::setElementsState $this"]
-        bind $path <ButtonRelease-1> "blt2DViewer::setSelection $this %x %y"
-        bind $path <Control-ButtonRelease-1> "blt2DViewer::toggleSelection $this %x %y"
-        bind $path <Shift-ButtonRelease-1> "blt2DViewer::extendSelection $this %x %y"
-
         set blt2DViewer::($this,colorIndex) 0
         set blt2DViewer::($this,path) $path
     }
 
     proc ~blt2DViewer {this} {
-        delete $blt2DViewer::($this,drag) $blt2DViewer::($this,drop) $blt2DViewer::($this,selector)
+        catch {delete $blt2DViewer::($this,drag)}
+        delete $blt2DViewer::($this,drop)
+        catch {delete $blt2DViewer::($this,selector)}
         eval delete $blt2DViewer::($this,elements)                                                       ;# delete existing elements
     }
 
@@ -63,11 +57,13 @@ if {$::officialBLT} {
     }
 
     proc validateDrag {this x y} {
-        if {([llength [selector::selected $blt2DViewer::($this,selector)]]>0)||([llength $blt2DViewer::($this,elements)]==0)} {
-            return 1                                                              ;# allow drag of selected elements or empty viewer
-        } else {
-            return 0                                                                                              ;# nothing to drag
+        if {[llength $blt2DViewer::($this,elements)]==0} {
+            return 1                                                                                   ;# allow drag of empty viewer
         }
+        # allow dragging if only from a selected cell
+        return [expr {\
+            [lsearch -exact [selector::selected $blt2DViewer::($this,selector)] [$blt2DViewer::($this,path) legend get @$x,$y]]>=0\
+        }]
     }
 
     proc monitorCell {this array row column} {
@@ -82,13 +78,21 @@ if {$::officialBLT} {
         set blt2DViewer::($this,colorIndex) [expr {($blt2DViewer::($this,colorIndex)+1)%[llength $blt2DViewer::(colors)]}]
         lappend blt2DViewer::($this,elements) $element                                                     ;# register new data line
         set blt2DViewer::($this,cell,$element) $cell
-        selector::add $blt2DViewer::($this,selector) $element
+        catch {selector::add $blt2DViewer::($this,selector) $element}               ;# selector may not exist if dragging disallowed
+    }
+
+    proc cells {this} {
+        set list {}
+        foreach element $blt2DViewer::($this,elements) {
+            lappend list $blt2DViewer::($this,cell,$element)
+        }
+        return $list
     }
 
     proc deletedElement {this array element} {
         viewer::unregisterTrace $this $array                                          ;# trace may no longer be needed on this array
         ldelete blt2DViewer::($this,elements) $element
-        selector::remove $blt2DViewer::($this,selector) $element
+        catch {selector::remove $blt2DViewer::($this,selector) $element}            ;# selector may not exist if dragging disallowed
         unset blt2DViewer::($this,cell,$element)
         if {[llength $blt2DViewer::($this,elements)]==0} {
             delete $this                                                            ;# self destruct when there are no more elements
@@ -150,4 +154,17 @@ if {$::officialBLT} {
         }
     }
 
+    proc allowDrag {this} {
+        set path $blt2DViewer::($this,path)
+
+        set blt2DViewer::($this,drag) [new dragSite -path $path -validcommand "blt2DViewer::validateDrag $this"]
+        dragSite::provide $blt2DViewer::($this,drag) OBJECTS "blt2DViewer::dragData $this"                ;# provide list of objects
+        dragSite::provide $blt2DViewer::($this,drag) DATACELLS "blt2DViewer::dragData $this"
+
+        set blt2DViewer::($this,selector) [new objectSelector -selectcommand "blt2DViewer::setElementsState $this"]
+        bind $path <ButtonRelease-1> "blt2DViewer::setSelection $this %x %y"
+        bind $path <Control-ButtonRelease-1> "blt2DViewer::toggleSelection $this %x %y"
+        bind $path <Shift-ButtonRelease-1> "blt2DViewer::extendSelection $this %x %y"
+    }
+
 }
