@@ -1,35 +1,57 @@
 # copyright (C) 1997-98 Jean-Luc Fontaine (mailto:jfontain@mygale.org)
-# this program is free software: please refer to the BSD type license enclosed in this package
+# this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: databar.tcl,v 1.9 1998/04/30 10:06:16 jfontain Exp $}
+set rcsId {$Id: databar.tcl,v 1.14 1998/08/01 11:16:16 jfontain Exp $}
 
 class dataBarChart {
 
     proc dataBarChart {this parentPath args} composite {
-        [new barchart $parentPath -title {} -bottommargin 6 -topmargin 3 -bufferelements 0 -plotborderwidth 1] $args
+        [new barchart $parentPath\
+            -title {} -bottommargin 6 -topmargin 3 -bufferelements 0 -plotborderwidth 1\
+            -plotbackground $widget::(default,ButtonBackgroundColor)\
+        ] $args
     } blt2DViewer {$widget::($this,path)} {
 if {$::officialBLT} {
         $widget::($this,path) grid off                                                                      ;# grid is on by default
-}
+        $widget::($this,path) xaxis configure -hide 1
+} else {
         $widget::($this,path) xaxis configure -mapped 0
+}
         composite::complete $this
     }
 
-    proc ~dataBarChart {this} {}
+    proc ~dataBarChart {this} {
+        if {[string length $composite::($this,-deletecommand)]>0} {
+            uplevel #0 $composite::($this,-deletecommand)                                   ;# always invoke command at global level
+        }
+    }
 
     proc options {this} {
         # force size values
         return [list\
+            [list -deletecommand {} {}]\
+            [list -draggable draggable Draggable 0 0]\
             [list -height height Height 200]\
             [list -mode mode Mode normal normal]\
             [list -width width Width 300]\
         ]
     }
 
+    proc set-deletecommand {this value} {}
+
     foreach option {-height -width} {
         proc set$option {this value} "\$widget::(\$this,path) configure $option \$value"
     }
 
+    proc set-draggable {this value} {
+        if {$composite::($this,complete)} {
+            error {option -draggable cannot be set dynamically}
+        }
+        if {$value} {
+            blt2DViewer::allowDrag $this
+        }
+    }
+
     proc set-mode {this value} {
         $widget::($this,path) configure -barmode $value
     }
