@@ -1,53 +1,75 @@
 # copyright (C) 1997-98 Jean-Luc Fontaine (mailto:jfontain@mygale.org)
-# this program is free software: please refer to the BSD type license enclosed in this package
+# this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: datapie.tcl,v 1.14 1998/04/21 16:20:36 jfontain Exp $}
+set rcsId {$Id: datapie.tcl,v 1.24 1998/07/12 21:46:50 jfontain Exp $}
 
 class dataPieChart {
 
-    proc dataPieChart {this parentPath width height thickness args} composite {[new canvas $parentPath] $args} viewer {} {
+    proc dataPieChart {this parentPath thickness args} composite {
+        [new canvas $parentPath -highlightthickness 0 -borderwidth 2] $args
+    } viewer {} {
         set path $widget::($this,path)
-
-        set dataPieChart::($this,pie) [new pie $path 0 0 $width $height\
-            -title {} -thickness $thickness -selectable 1\
-            -labeler [new piePeripheralLabeler $path -font $font::(mediumNormal) -smallfont $font::(smallNormal)]\
-        ]
         set dataPieChart::($this,slices) {}
-
         # allow dropping of data cells
         set dataPieChart::($this,drop) [new dropSite\
             -path $path -formats DATACELLS -command "viewer::view $this \$dragSite::data(DATACELLS)"\
         ]
-        set dataPieChart::($this,drag) [new dragSite -path $path -validcommand "dataPieChart::validateDrag $this"]
-        dragSite::provide $dataPieChart::($this,drag) OBJECTS "dataPieChart::dragData $this"
-        dragSite::provide $dataPieChart::($this,drag) DATACELLS "dataPieChart::dragData $this"
-
-        bind $path <Configure> "dataPieChart::resize $this"
 
         composite::complete $this
+
+        # wait till completion to create pie since -selectable option is not dynamically settable
+        set padding [$path cget -borderwidth]
+        set dataPieChart::($this,pie) [new pie $path $padding $padding\
+            -title {} -thickness $thickness -selectable $composite::($this,-draggable)\
+            -labeler [new piePeripheralLabeler $path\
+                -font $font::(mediumNormal) -smallfont $font::(smallNormal) -widestvaluetext {00.0 %}\
+            ]\
+        ]
+        set padding [expr {2*$padding}]                                      ;# width and height are diminished by twice the padding
+        bind $path <Configure>\
+            "switched::configure $dataPieChart::($this,pie) -width \[expr {%w-$padding}\] -height \[expr {%h-$padding}\]"
     }
 
     proc ~dataPieChart {this} {
-        delete $dataPieChart::($this,pie) $dataPieChart::($this,drag) $dataPieChart::($this,drop)
+        delete $dataPieChart::($this,pie) $dataPieChart::($this,drop)
+        catch {delete $dataPieChart::($this,drag)}
+        if {[string length $composite::($this,-deletecommand)]>0} {
+            uplevel #0 $composite::($this,-deletecommand)                                   ;# always invoke command at global level
+        }
     }
 
     proc options {this} {
         # force size values
         return [list\
+            [list -deletecommand {} {}]\
+            [list -draggable draggable Draggable 0 0]\
             [list -height height Height 200]\
             [list -width width Width 300]\
         ]
     }
 
+    proc set-deletecommand {this value} {}
+
+    proc set-draggable {this value} {
+        if {$composite::($this,complete)} {
+            error {option -draggable cannot be set dynamically}
+        }
+        if {!$value} return                                                                                           ;# no dragging
+        set dataPieChart::($this,drag) [new dragSite -path $widget::($this,path) -validcommand "dataPieChart::validateDrag $this"]
+        dragSite::provide $dataPieChart::($this,drag) OBJECTS "dataPieChart::dragData $this"
+        dragSite::provide $dataPieChart::($this,drag) DATACELLS "dataPieChart::dragData $this"
+    }
+
     foreach option {-height -width} {
         proc set$option {this value} "\$widget::(\$this,path) configure $option \$value"
     }
 
     proc dragData {this format} {
+        set slices [slice::selected $dataPieChart::($this,pie)]
         switch $format {
             OBJECTS {
-                if {[llength $dataPieChart::($this,selectedSlices)]>0} {
-                    return $dataPieChart::($this,selectedSlices)                          ;# return selected slices if there are any
+                if {[llength $slices]>0} {
+                    return $slices                                                        ;# return selected slices if there are any
                 } elseif {[llength $dataPieChart::($this,slices)]==0} {
                     return $this                                                       ;# return pie itself if it contains no slices
                 } else {
@@ -55,19 +77,19 @@ class dataPieChart {
                 }
             }
             DATACELLS {
-                return [cellsFromSlices $this $dataPieChart::($this,selectedSlices)]
+                return [cellsFromSlices $this $slices]
             }
         }
     }
 
-    # used to record selected slices before button release event during drag which eventually deselects selected slice
     proc validateDrag {this x y} {
-        set dataPieChart::($this,selectedSlices) [slice::selected $dataPieChart::($this,pie)]
-        if {([llength $dataPieChart::($this,selectedSlices)]>0)||([llength $dataPieChart::($this,slices)]==0)} {
-            return 1                                                                ;# allow drag of selected slices or empty viewer
-        } else {
-            return 0                                                                                              ;# nothing to drag
+        if {[llength $dataPieChart::($this,slices)]==0} {
+            return 1                                                                                   ;# allow drag of empty viewer
         }
+        # allow dragging if only from a selected slice
+        return [expr {\
+            [lsearch -exact [slice::selected $dataPieChart::($this,pie)] [slice::current $dataPieChart::($this,pie)]]>=0\
+        }]
     }
 
     proc supportedTypes {this} {
@@ -98,11 +120,6 @@ class dataPieChart {
                 slice::update $slice $value "[format %.1f [expr {$value*100}]] %"
             }
         }
-        resize $this
-    }
-
-    proc resize {this} {                                                                   ;# make sure all canvas items are visible
-        $widget::($this,path) configure -scrollregion [$widget::($this,path) bbox all]
     }
 
     proc deletedSlice {this array slice} {
@@ -121,6 +138,14 @@ class dataPieChart {
         }
         return $cells
     }
+
+    proc cells {this} {
+        set cells {}
+        foreach slice $dataPieChart::($this,slices) {
+            lappend cells $dataPieChart::($this,cell,$slice)
+        }
+        return $cells
+    }
 }
 
 class dataPieChart {
@@ -163,13 +188,22 @@ class dataPieChart {
             return $list
         }
 
+        proc current {pie} {                          ;# return current object (whose canvas slice is under the mouse cursor) if any
+            set slice [pie::currentSlice $pie]
+            if {$slice==0} {
+                return 0                                                                                         ;# no current slice
+            } else {
+                return [set (this,$slice)]                                                 ;# return object corresponding with slice
+            }            
+        }
+
     }
 
 }
 
 class data2DPieChart {
 
-    proc data2DPieChart {this parentPath args} dataPieChart {$parentPath 160 160 0 -width 200 -height 220 $args} {}
+    proc data2DPieChart {this parentPath args} dataPieChart {$parentPath 0 -width 200 -height 220 $args} {}
 
     proc ~data2DPieChart {this} {}
 
@@ -196,7 +230,7 @@ class data2DPieChart {
 
 class data3DPieChart {
 
-    proc data3DPieChart {this parentPath args} dataPieChart {$parentPath 200 100 20 -width 250 -height 180 $args} {}
+    proc data3DPieChart {this parentPath args} dataPieChart {$parentPath 20 -width 250 -height 180 $args} {}
 
     proc ~data3DPieChart {this} {}
 
