@@ -1,10 +1,12 @@
 # copyright (C) 1997-98 Jean-Luc Fontaine (mailto:jfontain@mygale.org)
-# this program is free software: please refer to the BSD type license enclosed in this package
+# this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: datatab.tcl,v 1.30 1998/04/29 22:52:34 jfontain Exp $}
+set rcsId {$Id: datatab.tcl,v 1.43 1998/08/01 19:19:36 jfontain Exp $}
 
 class dataTable {                       ;# module data view in the form of a table, that can be sorted using a column as a reference
 
+    set dataTable::(list) {}
+
     set dataTable::(scrollbarBorderWidth) [expr {$widget::(default,ScrollbarBorderWidth)==0?0:1}]
     set dataTable::(scrollbarWidth) [expr {2*$widget::(default,ScrollbarWidth)/3}]
 
@@ -24,15 +26,6 @@ class dataTable {                       ;# module data view in the form of a tab
 
         bindtags $path "$path . all"           ;# remove all class bindings for we do not use any and they would cause interferences
 
-        # allow dragging from data array
-        set dataTable::($this,drag) [new dragSite -path $path -validcommand "dataTable::validateDrag $this"]
-        dragSite::provide $dataTable::($this,drag) DATACELLS "dataTable::dragData $this"
-
-        set dataTable::($this,selector) [new tableSelector -selectcommand "dataTable::setCellsState $this"]
-        bind $path <ButtonRelease-1> "dataTable::buttonRelease $this %x %y"
-        bind $path <Control-ButtonRelease-1> "dataTable::toggleSelection $this %x %y"
-        bind $path <Shift-ButtonRelease-1> "dataTable::extendSelection $this %x %y"
-
         # allow border resizing with first button. does not interfere with drag bindings since command does nothing unless mouse
         # click occured in a column border, which cannot be the case when dragging (see drag validation procedure in this class)
         bind $path <ButtonPress-1> "dataTable::buttonPress $this %x %y"
@@ -40,7 +33,11 @@ class dataTable {                       ;# module data view in the form of a tab
 
         set dataTable::($this,sortOrder) increasing
         set dataTable::($this,tablePath) $path
+
+        lappend dataTable::(list) $this
+
         composite::complete $this
+        setupDataView $this                                              ;# wait till -data and eventually -view options are defined
     }
 
     proc ~dataTable {this} {
@@ -48,14 +45,18 @@ class dataTable {                       ;# module data view in the form of a tab
 
         catch {unset ${this}data}                                                                      ;# eventually free table data
         catch {eval delete $dataTable::($this,arrow) $dataTable::($this,tips)}
-        delete $dataTable::($this,drag) $dataTable::($this,selector)
+        catch {delete $dataTable::($this,drag)}
+        catch {delete $dataTable::($this,selector)}
+        ldelete dataTable::(list) $this
     }
 
     proc options {this} {
         return [list\
             [list -columns columns Columns 0 0]\
-            [list -titlefont titleFont TitleFont $font::(mediumBold) $font::(mediumBold)]\
             [list -data data Data {} {}]\
+            [list -draggable draggable Draggable 0 0]\
+            [list -titlefont titleFont TitleFont $font::(mediumBold) $font::(mediumBold)]\
+            [list -view view View {} {}]\
         ]
     }
 
@@ -65,29 +66,45 @@ class dataTable {                       ;# module data view in the form of a tab
 
     proc set-titlefont {this value} {
         if {[string length $composite::($this,-data)]==0} return                                         ;# nothing to do if no data
-        upvar $composite::($this,-data) data
 
         set path $dataTable::($this,tablePath)
-        if {![winfo exists $path.0]} {
-            createTitles $this
-        }
-        for {set column 0} {$column<$data(columns)} {incr column} {
+        for {set column 0} {$column<[llength $dataTable::($this,dataColumns]} {incr column} {
             $path.$column.label configure -font $value
         }
     }
 
     proc set-data {this value} {                                                ;# value must be a fully qualified module data array
-        upvar $value data
-
+        if {$composite::($this,complete)} {
+            error {option -data cannot be set dynamically}
+        }
         if {![array exists $value]} {
             error "\"$value\" argument is not an existing array"
         }
-        if {![winfo exists $dataTable::($this,tablePath).0]} {
-            createTitles $this
+    }
+
+    proc set-draggable {this value} {
+        if {$composite::($this,complete)} {
+            error {option -draggable cannot be set dynamically}
+        }
+        if {!$value} return                                                                                           ;# no dragging
+        set path $dataTable::($this,tablePath)
+        set dataTable::($this,drag) [new dragSite -path $path -validcommand "dataTable::validateDrag $this"]
+        dragSite::provide $dataTable::($this,drag) DATACELLS "dataTable::dragData $this"
+
+        set dataTable::($this,selector) [new tableSelector -selectcommand "dataTable::setCellsState $this"]
+        bind $path <ButtonRelease-1> "dataTable::buttonRelease $this %x %y"
+        bind $path <Control-ButtonRelease-1> "dataTable::toggleSelection $this %x %y"
+        bind $path <Shift-ButtonRelease-1> "dataTable::extendSelection $this %x %y"
+    }
+
+    # override default view defined in -data option with visibleColumns and sort members
+    proc set-view {this value} {                                            ;# value must be a fully qualified module sub data array
+        if {$composite::($this,complete)} {
+            error {option -view cannot be set dynamically}
+        }
+        if {![array exists $value]} {
+            error "\"$value\" argument is not an existing array"
         }
-        set dataTable::($this,sortColumn) [lindex $data(sort) 0]
-        set dataTable::($this,sortOrder) [lindex $data(sort) 1]
-        trace variable ${value}(updates) w "dataTable::update $this"                                    ;# track module data updates
     }
 
     proc buttonRelease {this x y} {
@@ -96,7 +113,7 @@ class dataTable {                       ;# module data view in the form of a tab
             if {$number==0} return
             scan [$dataTable::($this,tablePath) index @$x,$y] %d,%d row column
             if {$row<0} return                                                                           ;# title row, nothing to do
-            selector::select $dataTable::($this,selector) $row,$column
+            catch {selector::select $dataTable::($this,selector) $row,$column}      ;# selector may not exist if dragging disallowed
         }
         unset dataTable::($this,columnBorderHit)
     }
@@ -112,15 +129,14 @@ class dataTable {                       ;# module data view in the form of a tab
             set dataTable::($this,sortColumn) $column
             set dataTable::($this,sortOrder) increasing
         }
-        selector::clear $dataTable::($this,selector)       ;# deselect all cells since reordering rows renders selection meaningless
+        # deselect all cells since reordering rows renders selection meaningless
+        catch {selector::clear $dataTable::($this,selector)}                        ;# selector may not exist if dragging disallowed
         update $this                                                                                     ;# update table immediately
     }
 
     proc update {this args} {                                   ;# update display using module data. ignore eventual trace arguments
-        if {[string length $composite::($this,-data)]==0} return                                         ;# nothing to do if no data
-
         variable ${this}data
-        upvar $composite::($this,-data) data
+        upvar #0 $composite::($this,-data) data                                              ;# data must be visible at global level
 
         set path $dataTable::($this,tablePath)
 
@@ -130,32 +146,45 @@ class dataTable {                       ;# module data view in the form of a tab
 
         set sortColumn $dataTable::($this,sortColumn)
         set lists {}
-        foreach name [array names data *,0] {
-            scan $name %u dataRow
-            lappend lists [list $dataRow $data($dataRow,$sortColumn)]
+        if {[regexp {^integer|real$} $data($sortColumn,type)]} {                                                     ;# numeric type
+            foreach name [array names data *,$sortColumn] {
+                scan $name %u dataRow
+                if {[catch {expr {double($data($dataRow,$sortColumn))}}]} {                   ;# handle empty values or ? characters
+                    lappend lists [list $dataRow 0]                                                          ;# assume 0 for sorting
+                } else {
+                    lappend lists [list $dataRow $data($dataRow,$sortColumn)]
+                }
+            }
+        } else {
+            foreach name [array names data *,$sortColumn] {
+                scan $name %u dataRow
+                lappend lists [list $dataRow $data($dataRow,$sortColumn)]
+            }
         }
         # sort data rows according to sort column (column numbering is identical for table data and source data)
         set lists [lsort -$dataTable::($this,sortOrder) -$data($sortColumn,type) -index 1 $lists]
 
-        set selector $dataTable::($this,selector)
-        set columns $data(columns)
+        catch {set selector $dataTable::($this,selector)}                           ;# selector may not exist if dragging disallowed
         set changed 0                                                        ;# keep track of whether any rows were added or removed
         set row 0
         set rows {}
         foreach pair $lists {
             set dataRow [lindex $pair 0]
-            set exists [info exists ${this}data($row,-1)]
-            if {![info exists ${this}data($row,-1)]} {                                                            ;# gather new rows
+            if {![info exists ${this}data($row,dataRow)]} {                                                       ;# gather new rows
                 lappend rows $row
             }
-            set ${this}data($row,-1) $dataRow                                             ;# keep track of table / data rows mapping
-            for {set column 0} {$column<$columns} {incr column} {
-                set ${this}data($row,$column) $data($dataRow,$column)
+            set ${this}data($row,dataRow) $dataRow                                        ;# keep track of table / data rows mapping
+            set column 0
+            foreach dataColumn $dataTable::($this,dataColumns) {
+                set ${this}data($row,$column) $data($dataRow,$dataColumn)
+                incr column
             }
             incr row
         }
         $path configure -rows [expr {$row+1}]                                           ;# fit to data (take into account title row)
 
+        set columns [llength $dataTable::($this,dataColumns)]
+
         if {[llength $rows]>0} {                                                                      ;# one or more rows were added
             set changed 1
             set cells {}
@@ -164,11 +193,11 @@ class dataTable {                       ;# module data view in the form of a tab
                     lappend cells $new,$column
                 }
             }
-            selector::add $selector $cells                                               ;# make selector aware of new cells at once
+            catch {selector::add $selector $cells}                                       ;# make selector aware of new cells at once
         }
 
         set rows {}
-        while {[info exists ${this}data($row,-1)]} {                                                          ;# gather removed rows
+        while {[info exists ${this}data($row,dataRow)]} {                                                     ;# gather removed rows
             lappend rows $row
             incr row
         }
@@ -176,21 +205,19 @@ class dataTable {                       ;# module data view in the form of a tab
             set changed 1
             set cells {}
             foreach old $rows {
-                unset ${this}data($old,-1)
+                unset ${this}data($old,dataRow)
                 for {set column 0} {$column<$columns} {incr column} {
                     lappend cells $old,$column
                     unset ${this}data($old,$column)
                 }
             }
-            selector::remove $selector $cells                                        ;# make selector aware of removed cells at once
+            catch {selector::remove $selector $cells}                                ;# make selector aware of removed cells at once
         }
 
         if {$changed} {
-           selector::clear $selector                   ;# deselect all cells since new or deleted rows renders selection meaningless
+           catch {selector::clear $selector}           ;# deselect all cells since new or deleted rows renders selection meaningless
         }
 
-        updatedSortingArrow $this
-
         $path configure -cursor $cursor                                                                            ;# restore cursor
         ::update idletasks
     }
@@ -202,7 +229,8 @@ class dataTable {                       ;# module data view in the form of a tab
         set list {}
         foreach cell [selector::selected $dataTable::($this,selector)] {
             scan $cell %d,%d row column
-            lappend list ${data}([set ${this}data($row,-1)],$column)                        ;# data cell format is array(row,column)
+            # data cell format is array(row,column)
+            lappend list ${data}([set ${this}data($row,dataRow)],[set ${this}data($column,dataColumn)])
         }
         return $list
     }
@@ -211,11 +239,10 @@ class dataTable {                       ;# module data view in the form of a tab
         if {$dataTable::($this,columnBorderHit)} {
             return 0                                                                              ;# resizing a column: prevent drag
         }
-        if {[llength [selector::selected $dataTable::($this,selector)]]>0} {
-            return 1                                                                                   ;# at least one selected cell
-        } else {
-            return 0                                                                                              ;# nothing to drag
-        }
+        # allow dragging if only from a selected cell
+        return [expr\
+            {[lsearch -exact [selector::selected $dataTable::($this,selector)] [$dataTable::($this,tablePath) index @$x,$y]]>=0}\
+        ]
     }
 
     proc setCellsState {this cells select} {
@@ -240,9 +267,8 @@ class dataTable {                       ;# module data view in the form of a tab
         selector::extend $dataTable::($this,selector) $cell
     }
 
-    proc updatedSortingArrow {this} {
+    proc updateSortingArrow {this column} {
         set path $widget::($dataTable::($this,arrow),path)
-        set column $dataTable::($this,sortColumn)
 
         set label $dataTable::($this,tablePath).$column.label      ;# copy title label bindings for contextual help and mouse action
         foreach event {<Enter> <Leave> <ButtonRelease-1>} {
@@ -258,25 +284,32 @@ class dataTable {                       ;# module data view in the form of a tab
     }
 
     proc createTitles {this} {
-        upvar $composite::($this,-data) data
+        variable ${this}data
+        upvar #0 $composite::($this,-data) data                                              ;# data must be visible at global level
 
         set path $dataTable::($this,tablePath)
         set font $composite::($this,-titlefont)
-        for {set column 0} {$column<$data(columns)} {incr column} {                 ;# create table title labels in separate windows
+        set column 0
+        foreach dataColumn $dataTable::($this,dataColumns) {
+            # create table title labels in separate windows
             set frame [frame $path.$column]                       ;# use a frame as a container for label and eventual sorting arrow
-            set label [label $path.$column.label -font $font -text $data($column,label)]
+            # force default arrow cursor as column resizing cursor sticks when moving across columns
+            set label [label $path.$column.label -font $font -text $data($dataColumn,label) -cursor top_left_arrow]
             place $label -relwidth 1 -relheight 1         ;# use placer so that sorting arrow can eventually be displayed over label
             $path window configure -1,$column -window $frame -padx 2 -pady 2 -sticky nsew
-            bind $label <ButtonRelease-1> "dataTable::columnSort $this $column"
+            bind $label <ButtonRelease-1> "dataTable::columnSort $this $dataColumn; dataTable::updateSortingArrow $this $column"
             # setup context sensitive help on titles using help strings from module data
-            bind $label <Enter> "lifoLabel::push $::messenger [list $data($column,message)]"
+            bind $label <Enter> "lifoLabel::push $::messenger [list $data($dataColumn,message)]"
             bind $label <Leave> "lifoLabel::pop $::messenger"
             lappend dataTable::($this,tips) [new widgetTip -path $label -text {click to toggle sort}]
+            incr column
         }
         $path configure -cols $column                                                              ;# fit table to number of columns
         if {![info exists dataTable::($this,arrow)]} {                                    ;# use 1 sorting arrow indicator per table
             set arrow [new arrowButton $path -state disabled -borderwidth 0 -highlightthickness 0 -width 12]
             widget::configure $arrow -disabledforeground [widget::cget $arrow -foreground]               ;# make arrow fully visible
+            # force default arrow cursor as column resizing cursor sticks when moving across columns
+            $widget::($arrow,path) configure -cursor top_left_arrow
             lappend dataTable::($this,tips) [new widgetTip -path $widget::($arrow,path) -text {click to toggle sort}]
             set dataTable::($this,arrow) $arrow
         }
@@ -287,4 +320,39 @@ class dataTable {                       ;# module data view in the form of a tab
         set dataTable::($this,columnBorderHit) [expr {[info exists column]&&([string length $column]>0)}]
     }
 
+    proc setupDataView {this} {
+        if {[string length $composite::($this,-data)]==0} return                                         ;# nothing to do if no data
+
+        variable ${this}data
+        if {[string length $composite::($this,-view)]>0} {
+            upvar #0 $composite::($this,-view) data                                          ;# data must be visible at global level
+        } else {
+            upvar #0 $composite::($this,-data) data                                          ;# data must be visible at global level
+        }
+        if {[catch {set columns $data(visibleColumns)}]} {               ;# if not user defined visibility, make all columns visible
+            set columns {}                                                                                  ;# gather column indices
+            foreach name [array names data *,label] {
+                if {[scan $name %u column]>0} {
+                    lappend columns $column
+                }
+            }
+        }
+        set dataTable::($this,dataColumns) [lsort -integer $columns]                                     ;# then sort and store them
+        set dataTable::($this,sortColumn) [lindex $data(sort) 0]
+        if {[lsearch -exact $columns $dataTable::($this,sortColumn)]<0} {
+            error "sort column $dataTable::($this,sortColumn) is not visible"
+        }
+        set dataTable::($this,sortOrder) [lindex $data(sort) 1]
+        set column 0
+        foreach dataColumn $dataTable::($this,dataColumns) {                                   ;# store table / data columns mapping
+            set ${this}data($column,dataColumn) $dataColumn
+            if {$dataColumn==$dataTable::($this,sortColumn)} {
+                set sortColumnIndex $column
+            }
+            incr column
+        }
+        createTitles $this
+        updateSortingArrow $this $sortColumnIndex
+        trace variable $composite::($this,-data)(updates) w "dataTable::update $this"                   ;# track module data updates
+    }
 }
