@@ -1,9 +1,30 @@
 # copyright (C) 1997-98 Jean-Luc Fontaine (mailto:jfontain@mygale.org)
-# this program is free software: please refer to the BSD type license enclosed in this package
+# this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: drag.tcl,v 1.17 1998/04/20 14:37:43 jfontain Exp $}
+set rcsId {$Id: drag.tcl,v 1.20 1998/07/26 10:36:57 jfontain Exp $}
+
+### the following code should be moved to scwoop utilities and also used there in all related files ###
+if {[string compare $::tcl_platform(platform) windows]==0} {                    ;# compensate for win32 bug (thanks to Robin Becker)
+    proc showTopLevel {path geometry} {
+        wm deiconify $path
+        wm geometry $path $geometry
+    }
+} else {
+    proc showTopLevel {path geometry} {
+        wm geometry $path $geometry
+        wm deiconify $path
+    }
+}
 
 class dragSite {                                          ;# make a Tk widget a drag site with multiple formats support for its data
+
+    if {![info exists dragSite::(grabber)]} {
+        # use a specific invisible frame so that when dragging is active, the frame is grabbed and its specific cursor is used,
+        # thus preventing any interferences from source grab widget. use a specific cursor for user feedback
+        set dragSite::(grabber) $widget::([new frame . -background {} -width 0 -height 0 -cursor circle],path)
+        place $dragSite::(grabber) -x -32768 -y -32768                                               ;# make sure frame is invisible
+    }
+
     proc dragSite {this args} switched {$args} {
         switched::complete $this
     }
@@ -67,8 +88,7 @@ class dragSite {                                          ;# make a Tk widget a
     }
 
     proc hit {this xWidget yWidget xRoot yRoot} {
-        bindings::set $dragSite::($this,bindings) <ButtonRelease-1> {}                                             ;# reset bindings
-        bindings::set $dragSite::($this,bindings) <Button1-Motion> {}
+        bindings::set $dragSite::($this,bindings) <Button1-Motion> {}                                               ;# reset binding
         set command $switched::($this,-validcommand)
         if {([string length $command]>0)&&![uplevel #0 $command $xWidget $yWidget]} return
         set dragSite::(x) $xWidget                                ;# make widget and root coordinates available to provider commands
@@ -84,13 +104,7 @@ class dragSite {                                          ;# make a Tk widget a
         # do not actually start drag until mouse pointer is far enough, thus mimicking Windows behavior
         if {(abs($xRoot-$dragSite::(X))+abs($yRoot-$dragSite::(Y)))<5} return
 
-        set path $switched::($this,-path)
-        set bindings $dragSite::($this,bindings)
-
-        set dragSite::(bindtags) [bindtags $path]
-        bindtags $path bindings($bindings)    ;# temporarily ignore other bindings that may cause interferences (such as widget tip)
-
-        $path configure -cursor circle                                                    ;# use a specific cursor for user feedback
+        grab $dragSite::(grabber)                                                                 ;# drag cursor is used from now on
         update idletasks
 
         # for the highlighting frame, use a toplevel object to make sure its path is unique
@@ -100,8 +114,8 @@ class dragSite {                                          ;# make a Tk widget a
         set dragSite::(dropRegions) [dropSite::regions [array names ${this}provider]]      ;# note: drop code must not invoke update
         set dragSite::(lastSite) 0
         # setup bindings after initializations above
-        bindings::set $bindings <ButtonRelease-1> "dragSite::drop $this %X %Y"
-        bindings::set $bindings <Button1-Motion> "dragSite::track $this %X %Y"
+        bind $dragSite::(grabber) <ButtonRelease-1> "dragSite::drop $this %X %Y"
+        bind $dragSite::(grabber) <Button1-Motion> "dragSite::track $this %X %Y"
     }
 
     proc framed {x y left top right bottom} {                            ;# check whether a point is contained in a rectangular area
@@ -129,8 +143,7 @@ class dragSite {                                          ;# make a Tk widget a
             wm withdraw $frame                 ;# hide highlighting frame first so it shows no previously highlighted widget residue
             set path $switched::($site,-path)
             $frame configure -width [expr {[winfo width $path]+2}] -height [expr {[winfo height $path]+2}]
-            wm geometry $frame +[expr {[winfo rootx $path]-1}]+[expr {[winfo rooty $path]-1}]
-            wm deiconify $frame                                                                           ;# show highlighting frame
+            showTopLevel $frame +[expr {[winfo rootx $path]-1}]+[expr {[winfo rooty $path]-1}]            ;# show highlighting frame
         }
         set dragSite::(lastSite) $site
     }
@@ -139,10 +152,9 @@ class dragSite {                                          ;# make a Tk widget a
         variable ${this}provider
         variable data
 
-        bindtags $switched::($this,-path) $dragSite::(bindtags)                                         ;# restore original bindings
-        unset dragSite::(bindtags)
-
-        $switched::($this,-path) configure -cursor {}                                                     ;# restore original cursor
+        bind $dragSite::(grabber) <ButtonRelease-1> {}
+        bind $dragSite::(grabber) <Button1-Motion> {}
+        grab release $dragSite::(grabber)                                                          ;# cursor before grab is restored
         update idletasks
 
         delete $dragSite::(highlightFrame)
