@@ -1,7 +1,7 @@
 # copyright (C) 1997-98 Jean-Luc Fontaine (mailto:jfontain@mygale.org)
-# this program is free software: please refer to the BSD type license enclosed in this package
+# this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: html.tcl,v 1.11 1998/04/30 17:35:30 jfontain Exp $}
+set rcsId {$Id: html.tcl,v 1.13 1998/06/13 21:11:46 jfontain Exp $}
 
 # use bold and smaller sizes than the HTML library default ones for headers
 array set HMtag_map {
@@ -35,7 +35,7 @@ proc helpWindow {} {
         return
     }
     toplevel .topHelp
-    wm title .topHelp {moodss: global help}
+    wm title .topHelp {moodss: Global Help}
     frame .topHelp.bound                      ;# create a frame for bindings that otherwise would propagate to all toplevel children
 
     set panes [new panner .topHelp -panes 2]                                         ;# split window in 2 for contents and help data
