@@ -1,7 +1,7 @@
 # copyright (C) 1997-98 Jean-Luc Fontaine (mailto:jfontain@mygale.org)
-# this program is free software: please refer to the BSD type license enclosed in this package
+# this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: viewer.tcl,v 1.7 1998/04/30 20:54:34 jfontain Exp $}
+set rcsId {$Id: viewer.tcl,v 1.12 1998/07/11 18:09:44 jfontain Exp $}
 
 class viewer {                                                                             ;# handle viewers related functionalities
     # viewers do not derive from a common interface class but rather support a common set a options through the composite class
@@ -9,7 +9,6 @@ class viewer {
     set viewer::(list) {}
 
     proc viewer {this} {
-        new canvasHandles $::canvas -path $widget::($this,path)                                       ;# manage new viewer in canvas
         lappend viewer::(list) $this
     }
 
@@ -41,9 +40,6 @@ class viewer {
         if {[llength $cells]==0} return
         set parsed {}
         foreach cell $cells {
-            if {![info exists $cell]} {
-                error "\"$cell\" cell does not exist"
-            }
             parse $cell array row column type
             if {[lsearch -exact [supportedTypes $this] $type]<0} {
                 lifoLabel::flash $::messenger "cannot display data of type $type"
@@ -73,7 +69,11 @@ class viewer {
     proc label {array row column} {   ;# derive a label from cell using the tabular data index columns. array name must be qualified
         set label {}
         foreach index [set ${array}(indexColumns)] {                                                  ;# use index columns for label
-            append label "[set ${array}($row,$index)] "
+            if {[catch {set ${array}($row,$index)} value]} {
+                append label {? }               ;# cell may no longer exist if originating from a save file, give user some feedback
+            } else {
+                append label "$value "
+            }
         }
         append label [set ${array}($column,label)]
         return $label
@@ -99,4 +99,7 @@ class viewer {
             unset ${this}traces($array)
         }
     }
+
+    virtual proc cells {this}
+
 }
